"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dayJsPlugin = exports.formatWithCmdStr = exports.toDayjsDuration = exports.toDayjs = void 0;
const dayjs_1 = require("dayjs");
const duration_1 = require("dayjs/plugin/duration");
const utc_1 = require("dayjs/plugin/utc");
const localizedFormat_1 = require("dayjs/plugin/localizedFormat");
const core_1 = require("@fullcalendar/core");
dayjs_1.default.extend(localizedFormat_1.default);
dayjs_1.default.extend(duration_1.default);
dayjs_1.default.extend(utc_1.default);
function toDayjs(date, calendar) {
    if (!(calendar.render)) {
        throw new Error('must supply a CalendarApi instance');
    }
    const { dateEnv } = calendar.getCurrentData();
    return convertToDayjs(date, dateEnv.timeZone, null, dateEnv.locale.codes[0]);
}
exports.toDayjs = toDayjs;
function toDayjsDuration(fcDuration) {
    return dayjs_1.default.duration(fcDuration);
}
exports.toDayjsDuration = toDayjsDuration;
function formatWithCmdStr(cmdStr, arg) {
    const cmd = parseCmdStr(cmdStr);
    if (arg.end) {
        const startMom = convertToDayjs(arg.start.array, arg.timeZone, arg.start.timeZoneOffset, arg.localeCodes[0]);
        const endMom = convertToDayjs(arg.end.array, arg.timeZone, arg.end.timeZoneOffset, arg.localeCodes[0]);
        return formatRange(cmd, createDayjsFormatFunc(startMom), createDayjsFormatFunc(endMom), arg.defaultSeparator);
    }
    return convertToDayjs(arg.date.array, arg.timeZone, arg.date.timeZoneOffset, arg.localeCodes[0]).format(cmd.whole);
}
exports.formatWithCmdStr = formatWithCmdStr;
exports.dayJsPlugin = (0, core_1.createPlugin)({
    name: 'dayjsPlugin',
    cmdFormatter: formatWithCmdStr,
});
exports.default = exports.dayJsPlugin;
function createDayjsFormatFunc(mom) {
    return (cmdStr) => (cmdStr ? mom.format(cmdStr) : '' // because calling with blank string results in ISO8601 :(
    );
}
function convertToDayjs(input, timeZone, timeZoneOffset, locale) {
    let mom;
    if (timeZone === 'local') {
        mom = (0, dayjs_1.default)(input);
    }
    else if (timeZone === 'UTC') {
        mom = dayjs_1.default.utc(input);
    }
    else if (dayjs_1.default.tz) {
        mom = dayjs_1.default.tz(input, timeZone);
    }
    else {
        mom = dayjs_1.default.utc(input);
        if (timeZoneOffset != null) {
            mom.utcOffset(timeZoneOffset);
        }
    }
    mom.locale(locale);
    return mom;
}
function parseCmdStr(cmdStr) {
    const parts = cmdStr.match(/^(.*?)\{(.*)\}(.*)$/); // TODO: lookbehinds for escape characters
    if (parts) {
        const middle = parseCmdStr(parts[2]);
        return {
            head: parts[1],
            middle,
            tail: parts[3],
            whole: parts[1] + middle.whole + parts[3],
        };
    }
    return {
        head: null,
        middle: null,
        tail: null,
        whole: cmdStr,
    };
}
function formatRange(cmd, formatStart, formatEnd, separator) {
    if (cmd.middle) {
        const startHead = formatStart(cmd.head);
        const startMiddle = formatRange(cmd.middle, formatStart, formatEnd, separator);
        const startTail = cmd.tail;
        const endHead = formatEnd(cmd.head);
        const endMiddle = formatRange(cmd.middle, formatStart, formatEnd, separator);
        const endTail = cmd.tail;
        if (startHead === endHead && startTail === endTail) {
            return startHead +
                (startMiddle === endMiddle ? startMiddle : startMiddle + separator + endMiddle) +
                startTail;
        }
    }
    const startWhole = formatStart(cmd.whole);
    const endWhole = formatEnd(cmd.whole);
    if (startWhole === endWhole) {
        return startWhole;
    }
    return startWhole + separator + endWhole;
}
//# sourceMappingURL=fullcalendar-dayjs.js.map